 /*
 ==========Arduino Binary Alarm Clock==========
 | Author: Martin Forsgren                    |
 | Date:   15-11-2009                         |
 ==============================================
 */
 
#include <CapSense.h> // library for touchsensors
#include <Tone.h>    // library for soundgeneration
#define DEBUG 0     // set to 1 to get debugging output via serial.

// Display pins:
#include "WProgram.h"
void setup();
void loop();
void clock();
void ifClock();
void display();
void alarm();
void update_buttons_state();
void buttons();
void play_melody();
void update_buttons_state_with_debounce();
int hour_pins[]   = {18, 17, 16, 15, 14};    // hour pins are pins 14 to 18
int minute_pins[] = {13, 12, 11, 10, 9, 8}; // minute pins are 8 to 13
// the values are in falling order because I connected the pins that way by accident

// Button pins:
int hour_button_pin = 6;
int minute_button_pin = 5;
int setting_switch_pin = 7;

// Touch sensor:
// pin 4 is send pin, pin 2 is recieve pin (where the foil is attatched)
CapSense touch_sensor = CapSense(4, 2);

// Sound:
int speaker_pin = 19;
Tone tone_maker;

// Button states:
int hour_button_state;
int minute_button_state;
int setting_switch_state;
int snooze_button_state;

// Clock variables, that stores the current time:
int seconds = 0;
int minutes = 10;
int hours = 10;

// hours_p and minutes_p are pointers that are used to decide: 
// - if the ordinary time or the alarm time shoud be displayed
// - if the time or the alarm time should be changed when the buttons are pressed
int * hours_p = &hours;     // points to hours or alarm_hours depending 
                            // on the setting_switch_state
int * minutes_p = &minutes; // points to minutes or alarm_minutes depending 
                            // on the setting_switch_state

// ALARM VARIABLES:
boolean  alarm_on = false;
boolean signal_on = false;
boolean snooze_on = false;
// alarm time:
int alarm_hours = 20;
int alarm_minutes = 20;
// time when snooze period should end:
int snooze_off_hours;
int snooze_off_minutes;

int debug_print_counter = 0;

void setup()
{
  
  // set all six minute pins to output: 
  for(int i = 0; i < 7; i++)
  {
    pinMode(minute_pins[i], OUTPUT);
  } 
  
  // set all five hour pins to output:
  for(int i = 0; i < 6; i++) 
  {
    pinMode(hour_pins[i], OUTPUT);
  }

  // set the button pins to input:
  pinMode(hour_button_pin, INPUT);
  pinMode(minute_button_pin, INPUT);
  pinMode(setting_switch_pin, INPUT);
  // activate the internal pullup resistors:
  digitalWrite(hour_button_pin, HIGH);
  digitalWrite(minute_button_pin, HIGH);
  digitalWrite(setting_switch_pin, HIGH);
  
  //output sound to the speaker_pin:
  tone_maker.begin(speaker_pin);

  if(DEBUG)
  {
    Serial.begin(9600); // for debugging output.
  }
}

void loop() 
{
  clock();   // keep track of time, i.e. update hours,
             // minutes and seconds variables as needed.
  display(); // display the time, or the alarm time, depending on the state of the settings switch.
  alarm();   // checks if it's time for the alarm  to start.
  update_buttons_state(); // checks if the buttons and touch sensor states has changed
  //update_buttons_state_with_debounce();
  buttons(); // does what the buttons should do
  
  if(DEBUG)
  {
    if(debug_print_counter>100)
    {
      Serial.print("time: ");
      Serial.print(hours);
      Serial.print(":");
      Serial.print(minutes);
      Serial.print(":");
      Serial.println(seconds);
      Serial.print("alarm time: ");
      Serial.print(alarm_hours);
      Serial.print(":");
      Serial.println(alarm_minutes);
      
      Serial.print("hour_button_state: ");
      Serial.println(hour_button_state);
      Serial.print("CapSense value: ");
      Serial.println(touch_sensor.capSense(10));
      
      debug_print_counter=0;
    }
    else
    {
      debug_print_counter++;
    }
  }
}
//"INTERNAL" VARIABLES FOR CLOCK FUNCTION:
#define MAX_MILLIS_VALUE 34359738
unsigned long current_millis_value = 0;
unsigned long previous_millis_value = 0;
unsigned long m = 0;
/*
unsigned int seconds = 0;
unsigned int minutes = 0;
unsigned int hours = 0;
*/
void clock()
{
  current_millis_value = millis();
  if (current_millis_value < previous_millis_value) // if millis overflows
  {
    m += MAX_MILLIS_VALUE - previous_millis_value + current_millis_value;
  }
  else // if millis has not overflown
  {
    m += current_millis_value - previous_millis_value;
  }
  seconds += m / 1000;
  m = m % 1000;
  minutes += seconds / 60;
  seconds = seconds % 60;
  hours += minutes / 60;
  minutes = minutes % 60;
  hours = hours % 24;
  previous_millis_value = current_millis_value;
}

/* Clock function that uses if statements instead of modulo */
void ifClock()
{
  current_millis_value = millis();
  if (current_millis_value < previous_millis_value) // if millis overflows
  {
    m += MAX_MILLIS_VALUE - previous_millis_value + current_millis_value;
  }
  else // if millis has not overflown
  {
    m += current_millis_value - previous_millis_value;
  }
  if (m>999)
  {
    seconds++;
    m = m-1000;
  }
  if (seconds>59) // if seconds == 60
  {
    minutes++;
    seconds = 0;
  }
  if (minutes>59) // if minutes == 60
  {
    hours++;
    minutes = 0;
  }
  if (hours>23) // if hours == 24
  {
    hours = 0;
  }
  
  previous_millis_value = current_millis_value;
}
  
// "INTERNAL" VARIABLES FOR DISPLAY FUNCTION: 
int remainder;
int led_value;

void display()
{ 
  // display() will display the ordinary time or the alarm time,
  // depending on what hours_p points to, this is decided
  // by the settings_switch_state, in the buttons() function
  
  // Display hours:
  remainder = *hours_p;
  for(int i = 0; i < 5; i++) // repeat four all five hour-LEDs
  { 
    led_value = 16/round(pow(2,i)); // first LED = 16, second = 8, third = 4 etc.
    
    if(remainder/led_value == 1)
      digitalWrite(hour_pins[i], HIGH); 
    else
      digitalWrite(hour_pins[i], LOW);
    
    // the remainder of the hours is saved for 
    // the next LED that is displaying a lower value
    remainder=remainder%led_value;
  }
  
  // Display minutes:
  remainder = *minutes_p;
  for(int i = 0; i < 6; i++) // repeat for all six minute-LEDs
  { 
    led_value = 32/round(pow(2,i)); // first 32, then 16, then 8 etc.
    
    if(remainder/led_value == 1)
      digitalWrite(minute_pins[i], HIGH);
    else
      digitalWrite(minute_pins[i], LOW);
    
    // the remainder of the minutes is saved for 
    // the next LED that is displaying a lower value
    remainder=remainder%led_value;
  }
}
// "INTERNAL" VARIABLES FOR ALARM FUNCTION:
boolean first_time_signal_on = true; // used to make shure the signal is
                                    // only started once, so that you can
                                   // snooze without the alarm starting again
                                  // imidiately.

void alarm()
{
  if(alarm_on)
  {
    // Check if the time is same as alarm time:
    if(hours==alarm_hours && minutes==alarm_minutes && first_time_signal_on)
    {
      // if so, turn on the alarm signal:
      signal_on = true;
      first_time_signal_on = false;
    }
    
    if(signal_on)
    {
      play_melody();
    }
    
    // look in buttons() for snooze button and alarm off button
    
    if(snooze_on)
    {
      // Check if the time is same as snooze off time:
      if(hours==snooze_off_hours && minutes==snooze_off_minutes)
      {
        // if so, turn off snooze and restart alarm signal:
        snooze_on = false;
        signal_on = true;
      }
    }  
  }
  else
  {
    // reset so that the alarm will work next time:
    first_time_signal_on = true;
  }
}
void update_buttons_state()
{
  hour_button_state    = digitalRead(hour_button_pin);
  minute_button_state  = digitalRead(minute_button_pin); 
  setting_switch_state = digitalRead(setting_switch_pin);
  
  /* Read the snooze touch sensor: */
  if(touch_sensor.capSense(10) > 100) // adjust this value if needed
    snooze_button_state = LOW;
  else
    snooze_button_state = HIGH;
  // the if-else above I think could be written shorter as:
  // snooze_button_state = (touch_sensor.capSense(10) > 100) ? LOW : HIGH;
}


// "INTERNAL" VARIABLES FOR BUTTONS FUNCTION:
boolean first_time_hour = true;   // these are used to make sure that the hours
boolean first_time_minute = true; // and minutes only is increased once every keypress.
unsigned long snooze_button_timer; // used to keep track of how long the snooze button has
                                   // been held down. when the button has been held down
                                   // a certain amount of time, the alarm will be turned
                                   // of completely.
void buttons()
{
  // LOW == button pressed
  // HIGH == button released
  // (this is because pullup resistors is used)
  
  // Decide if we should set time or alarm:
  // (this also makes the display show the alarm time)
  if(setting_switch_state==LOW) // LOW = Set time
  {
    hours_p = &hours;
    minutes_p = &minutes;
  }
  else // HIGH = Set alarm
  {
    hours_p = &alarm_hours;
    minutes_p = &alarm_minutes;
  }

  // If hour button is pressed, increase hours:
  if(hour_button_state==LOW && first_time_hour) // only increase the hours once
  {                                             // every button press.
    if(*hours_p < 23)
      (*hours_p)++;
    else
      *hours_p = 0;
    
    first_time_hour = false;
    
    if(DEBUG)
    {
      Serial.println("hour increase");
      Serial.println(hour_button_state);
    }
  }
  else if(hour_button_state==HIGH)
  {
    first_time_hour = true; // reset when button is released, 
  }                         // so that the next press will be registerd.
  
  // If minute button is pressed, increase minutes:
  if(minute_button_state==LOW && first_time_minute) // only increase the minutes
  {                                                 // once every button press.
    if(*minutes_p < 59)
      (*minutes_p)++;
    else
      *minutes_p = 0;
      
    first_time_minute = false;
  }
  else if(minute_button_state==HIGH)
  {
    first_time_minute = true; // reset when button is released, 
  }                           // so that the next press will be registerd.

  
  if(snooze_button_state==LOW)
  {
    if(signal_on)
    {
      // set the time when the alarm signal will start again,
      // this will give 10 minutes snooze:
      if(minutes<50)
      {
        snooze_off_minutes = minutes+10; 
        snooze_off_hours = hours;
      }
      else
      {
        snooze_off_minutes = minutes - 50; 
        snooze_off_hours = hours + 1;
      }
      snooze_on = true;
      signal_on = false;
    }
    // if the snooze button has been held down for more than 3 seconds turn off/on the alarm
    if((millis() - snooze_button_timer) > 3000) 
    {
      if(alarm_on) // if on, turn off
      {
        signal_on = false; 
        alarm_on = false;
        snooze_on = false;
        // play tone so the user know the alarm turned off:
        tone_maker.play(NOTE_A3, 100);
      }
      else if(alarm_on==false) // if off, turn on
      {
        alarm_on = true;
        // play tone so the user know the alarm turned off:
        tone_maker.play(NOTE_A7, 100);
      }
      //reset the snooze button timer
      snooze_button_timer=millis();
    }
    
  }
  else
  {
    //reset the snooze button timer
    snooze_button_timer=millis();
  }
}




// "INTERNAL" VARIABLES FOR PLAY_MELODY FUNCTION:
int melody[] = { NOTE_C4, NOTE_D4, NOTE_E4, NOTE_F4, 
                 NOTE_G4, NOTE_A4, NOTE_B4, NOTE_C5 };
int melody_length = sizeof(melody) / 2; // Melody length, for looping.
// sizeof() returns the size of the array in bytes, and because 
// an int is 2 bytes, sizeof will return 2*(number of array elements)

int i = 0;        //loop variable
boolean reset_loop = true; 
int jump = 1;     // how many notes to jump in the melody array
int position = 0; // position in melody array

void play_melody()
{
  if(!(tone_maker.isPlaying())) // if the last tone has stopped
  {
    if(i<melody_length)
    {
      //tone_maker.stop();
      tone_maker.play(melody[position], 300);
      // A pause between notes...
      //delay(300); // replace with nonblocking.
      
      if (DEBUG) 
      { // If debugging, report loop, tone, beat, and duration
        Serial.print(position);
        Serial.print(":");
        Serial.print(melody[position]);
        Serial.print("\n");
      }
      
      position += jump;
      //if position is bigger than the array, start from beginning of array:
      position = position%melody_length; 
      i++;
     }
     else
     {
       jump++;
       i = 0;
     }
  }
}
#if 0  // this whole tab is outcommented



/**********************************************************
 * Here's a update_buttons_state function                 *
 * with debouncing for the hour and minute                *
 * buttons:                                               *
 * ( Debouncing code is based on the example found here:  *
 * http://www.arduino.cc/en/Tutorial/Debounce )           *  
 **********************************************************/
// "INTERNAL" VARIABLES: 
int hour_reading;
int last_hour_reading = HIGH;
int minute_reading;
int last_minute_reading = HIGH;
long last_debounce_time_hours = 0; // last time button state was changed
long last_debounce_time_minutes = 0;
long debounce_delay = 50;

void update_buttons_state_with_debounce()
{
  /* Hour button debouncing: */
  hour_reading=digitalRead(hour_button_pin);
  // If the button state has changed, due to noise or pressing:
  if (hour_reading != last_hour_reading) 
  {
    // reset the debouncing timer
    last_debounce_time_hours = millis();
  }
  // if the button state has been unchanged for long enough
  if ((millis() - last_debounce_time_hours) > debounce_delay)
  {
    // change the button state to the state read:
    hour_button_state=hour_reading;
  }
  // save the reading for later comparison 
  last_hour_reading = hour_reading;
  
  /* Minute button debouncing: */
  minute_reading=digitalRead(minute_button_pin);
  // If the button state has changed, due to noise or pressing:
  if (minute_reading != last_minute_reading) 
  {
    // reset the debouncing timer
    last_debounce_time_minutes = millis();
  }
  // if the button state has been unchanged for long enough
  if ((millis() - last_debounce_time_minutes) > debounce_delay)
  {
    // change the button state to the state read:
    minute_button_state=minute_reading;
  }
  // save the reading for later comparison 
  last_minute_reading = minute_reading;
  
  /* Read the setting switch */
  setting_switch_state = digitalRead(setting_switch_pin);
  
  /* Read the snooze touch sensor: */
  if(touch_sensor.capSense(10) > 60) // adjust this value if needed
    snooze_button_state = LOW;
  else
    snooze_button_state = HIGH;
}

#endif // end of comment

int main(void)
{
	init();

	setup();
    
	for (;;)
		loop();
        
	return 0;
}

